#include <math.h>

#include "attribute_norms.h"



static FLT_TYPE * norm_inv = NULL;



static void
inverse_norms(FLT_TYPE * attr_norms, size_t nattrs)
{
    size_t i;

    for(i = 0; i < nattrs; i++)
    {
	if(attr_norms[i] == (FLT_TYPE)0)
	{
	    attr_norms[i] = (FLT_TYPE)1;
	}
	else
	{
	    attr_norms[i] = (FLT_TYPE)1 / attr_norms[i];
	}
    }
}

void
init_attribute_normalization(FILE * f, size_t nattrs,
			     size_t * N0_p, size_t * N1_p,
			     enum norm_type norm)
{
    size_t i;
    int class;

    FLT_TYPE * X;
    size_t N0 = 0UL;
    size_t N1 = 0UL;


    norm_inv = malloc(nattrs * sizeof(FLT_TYPE));
    X = malloc(sizeof(FLT_TYPE) * nattrs);

    for(i = 0; i < nattrs; i++)
    {
	norm_inv[i] = (FLT_TYPE)0;
    }

    rewind(f);
    while((class = read_record(f, X, nattrs)) != -1)
    {
	if(class == 0)
	{
	    N0++;
	}
	else
	{
	    N1++;
	}

	for(i = 0; i < nattrs; i++)
	{
	    switch(norm)
	    {
	    case L1_NORM:
		norm_inv[i] += ABS_F(X[i]);
	    case L2_NORM:
		norm_inv[i] += ABS_F(X[i]) * ABS_F(X[i]);
	    case L_INF_NORM:
		if(ABS_F(X[i]) > norm_inv[i])
		{
		    norm_inv[i] = ABS_F(X[i]);
		}
		break;
	    case NO_NORM:
		break;
	    }
	}
    }

    if(norm == L2_NORM)
    {
	for(i = 0; i < nattrs; i++)
	{
	    norm_inv[i] = SQRT_F(norm_inv[i]);
	}
    }
    inverse_norms(norm_inv, nattrs);

    /* set return values */
    *N0_p = N0;
    *N1_p = N1;

    free(X);
}

void reset_attribute_normalization(size_t nattrs)
{
    size_t i;

    for(i = 0; i < nattrs; i++)
    {
	norm_inv[i] = (FLT_TYPE)1;
    }
}


void free_attribute_normalization(void)
{
    if(norm_inv != NULL)
    {
	free(norm_inv);
	norm_inv = NULL;
    }
}

void
normalize_vect(FLT_TYPE * X, size_t nattrs)
{
    size_t i;

    for(i = 0; i < nattrs; i++)
    {
	X[i] *= norm_inv[i];
    }
}

int
read_and_normalize_record(FILE * f, FLT_TYPE * X, size_t nattrs)
{
    int class;

    class = read_record(f, X, nattrs);
    normalize_vect(X, nattrs);
    return class;
}
