#ifndef __ATTRIBUTE_NORMS__
#define __ATTRIBUTE_NORMS__

#include <stdio.h>
#include <stdlib.h>

#include "auc_defs.h"
#include "data_access.h"


/* norm to use.  NO_NORM means no normalization. */
enum norm_type {NO_NORM, L1_NORM, L2_NORM, L_INF_NORM};

/* initializes internal weights for attribute normalization using the
   L infinity (max) norm. Additionally returns the number of records
   in each class. */
void
init_attribute_normalization(FILE * f, size_t nattrs,
			     size_t * N0_p, size_t * N1_p,
			     enum norm_type norm);
/* frees internal weights for attribute normalization */
void free_attribute_normalization(void);

/* set all norms to 1, disabling normalization */
void reset_attribute_normalization(size_t nattrs);


/* normalize given vector */
void
normalize_vect(FLT_TYPE * X, size_t nattrs);

/* like read record but also normalized attribute values.

init_attribute_normalization should be called first. */
int
read_and_normalize_record(FILE * f, FLT_TYPE * X, size_t nattrs);
/* sets normalization weights for compute_attribute_Linf_norm */

#endif  /* __ATTRIBUTE_NORMS__ */
