#include "cheb_expand.h"

FLT_TYPE *
cheb_expand(FLT_TYPE (*f)(FLT_TYPE), size_t maxdeg)
{
    size_t i, j;
    FLT_TYPE * res;
    FLT_TYPE * ftab;
    FLT_TYPE fac;

    res = malloc((maxdeg + 1) * sizeof(FLT_TYPE));
    ftab = malloc((maxdeg + 1) * sizeof(FLT_TYPE));
    for(i = 0; i <= maxdeg; i++)
    {
	res[i] = (FLT_TYPE)0;
	ftab[i] = (FLT_TYPE)0;
    }

    fac = (FLT_TYPE)2 / ((FLT_TYPE)maxdeg + (FLT_TYPE)1);

    for(i = 0; i <= maxdeg; i++)
    {
	FLT_TYPE y;

        y = COS_F(PI * ((FLT_TYPE)i + (FLT_TYPE)0.5) / (FLT_TYPE)(maxdeg + 1));
        ftab[i] = f(y);
    }

    for(i = 0; i <= maxdeg; i++)
    {
        FLT_TYPE sum = (FLT_TYPE)0;
	
	for(j = 0; j <= maxdeg; j++)
	{
            sum += ftab[j] * COS_F(PI * (FLT_TYPE)i * ((FLT_TYPE)j + (FLT_TYPE)0.5) / (FLT_TYPE)(maxdeg + 1));
	}
        res[i] = fac * sum;
    }

    free(ftab);
    res[0] /= 2;
    return res;
}
