#include "data_access.h"


/* buffer for reading lines */
static char linebuf[MAXLINE];



/* caching datafile in memory */
struct data_cache
{
    size_t nattrs;
    size_t cur_rec;		/* current record */
    size_t N;
    FLT_TYPE * data;		/* attribute values */
    int * classes;		/* class values */
};

static int use_def_cache = FALSE;
static struct data_cache default_cache;

void
cache_file(FILE * f, size_t nattrs)
{
    int class;
    size_t N_alloc = 1024UL;

    if(use_def_cache == TRUE)
    {
	free_default_cache();
    }

    default_cache.nattrs = nattrs;
    default_cache.N = 0UL;
    default_cache.data = malloc(N_alloc * nattrs * sizeof(FLT_TYPE));
    default_cache.classes = malloc(N_alloc * sizeof(int));
    rewind(f);
    while((class = read_record(f, default_cache.data + 
			       nattrs * default_cache.N,
			       nattrs)) != -1)
    {
	default_cache.classes[default_cache.N] = class;
	default_cache.N++;
	if(default_cache.N >= N_alloc)
	{
	    N_alloc += N_alloc / 4; /* be more conservative on mem here */
	    default_cache.data = realloc(default_cache.data, N_alloc * nattrs * sizeof(FLT_TYPE));
	    default_cache.classes = realloc(default_cache.classes, N_alloc * sizeof(int));
	}
    }
    default_cache.cur_rec = 0UL;
    use_def_cache = TRUE;
}
void
free_default_cache(void)
{
    if(use_def_cache == TRUE)
    {
	free(default_cache.data);
	free(default_cache.classes);
	use_def_cache = FALSE;
    }
}




size_t
get_nattrs(FILE * f)
{
    size_t nattrs = 0;
    char * cptr;

    cptr = fgets(linebuf, MAXLINE, f);
    rewind(f);
    if(cptr == NULL)
    {
	return 0UL;
    }
    while(*cptr != '\0')
    {
	if(*cptr++ == ',')
	{
	    nattrs++;
	}
    }
    return nattrs + 1;
}

int
read_record(FILE * f, FLT_TYPE * X, size_t nattrs)
{
    char * line, * field;
    size_t i = 0;
    int class = 2;

    if(use_def_cache == TRUE)
    {
	if(default_cache.cur_rec >= default_cache.N)
	{
	    default_cache.cur_rec = 0UL;
	    return -1;
	}
	for(i = 0; i < nattrs; i++)
	{
	    X[i] = default_cache.data[default_cache.cur_rec *
				      default_cache.nattrs + i];
	}
	class = default_cache.classes[default_cache.cur_rec];
	default_cache.cur_rec++;
	return class;
    }

    line = fgets(linebuf, MAXLINE, f);
    if(line == NULL)
    {
	class = -1;
    }
    else
    {
	while(i < nattrs)
	{
	    field = line;
	    while(*line != ',' && *line != '\0')
	    {
		line++;
	    }
	    if(*line == '\0')
	    {
		class = -1;
		break;
	    }
	    *line = '\0';
	    X[i++] = (FLT_TYPE)atof(field);
	    line++;
	}
	if(*line != '\0')
	{
	    class = atoi(line);
	}
    }
    return class;
}


