#ifndef __MINIM_ALONG_GRAD__
#define __MINIM_ALONG_GRAD__


#include <stdio.h>
#include <stdlib.h>

#include "auc_defs.h"
#include "vect_utils.h"
#include "scaling.h"
#include "attribute_norms.h"


/* structure for holding caching data for minimization along
   gradient */
struct minim_cache
{
    size_t maxdeg;
    FLT_TYPE * cmb;		/* combinations */
    FLT_TYPE * alphas;		/* alpha coefficients */
    FLT_TYPE * sincos;		/* cache of trigonometric functions
				   for each optimization step */
    FLT_TYPE * step_coeff;	/* remember the step function
				   chebyshev coefficients used. */

    /* supports for minim. along grad, rearranged in better order and
       pre-multiplied by cmb */
    FLT_TYPE * supports_fwfg0;
    FLT_TYPE * supports_fwfg1;
};



/* compute supports needed to minimize along the gradient.

  Pointers to support arrays are put into supports_fwfg[01]_ret.
  Scaling for fw and fg is put into scale_w and scale_g
  respectively.

  equal_scales means that the same scaling should be used for weights
  and for gradient
*/
void
supports_for_minim_along_grad(FLT_TYPE ** supports_fwfg0_ret,
			      FLT_TYPE ** supports_fwfg1_ret,
			      const FLT_TYPE * weights,
			      const FLT_TYPE * grad,
			      size_t nattrs, size_t maxdeg,
			      FILE * f,
			      FLT_TYPE * scale_w, FLT_TYPE * scale_g,
			      int equal_scales);



/* print the supports needed for minimization along gradient */
void
print_supports_fwfg(FLT_TYPE * supports_fwfg,
		    size_t nattrs, size_t maxdeg);


/* compute AUC at angle alpha, based on precumputed supports.
 lin_opt_step is the iteration number used to look-up sincos(alpha) in
 the cache. */
FLT_TYPE
compute_AUC_along_grad(FLT_TYPE * supports_fwfg0,
		       FLT_TYPE * supports_fwfg1,
		       const struct minim_cache * mc,
		       FLT_TYPE alpha,
		       size_t nattrs, size_t maxdeg,
		       size_t lin_opt_step);


/* caching */

void
init_minim_cache(struct minim_cache * mc, size_t maxdeg,
		 FLT_TYPE lin_opt_step, FLT_TYPE * step_coeff);

void
free_minim_cache(struct minim_cache * mc);


void
precompute_alphas(struct minim_cache * mc, FLT_TYPE * step_coeff,
		  size_t maxdeg);

void minim_cache_set_supports(struct minim_cache * mc,
			      FLT_TYPE * supports_fwfg0,
			      FLT_TYPE * supports_fwfg1,
			      size_t maxdeg);


#endif  /* __MINIM_ALONG_GRAD__ */
