#include <string.h>

#include "opt_parse.h"

static void
print_usage_and_exit(char * prog_name)
{
    printf("usage: %s [options] training_file_name [test_file_name]\n"\
"-d deg       degree of polynomials to use [30]\n"\
"-i maxiter   maximum number of iterations [20]\n"\
"-n norm      norm to use for attr. normalization {none, L1, L2, max} [max]\n"\
"-w           print resulting weights\n"\
"-c           don't cache dataset in memory (for large datasets)\n"\
"-e           compute AUC exactly\n"\
"-f           use soft-AUC\n"\
"-b beta      the value of beta for soft-AUC [1.0]\n"\
"-s step      step to use in linear (circular) optimization [0.01]\n",
	   prog_name);
    exit(10);
}

void
parse_options(struct options * opt, int argc, char * argv[])
{
    int i;

    /* initialize fields */
    opt -> train_fname = NULL;
    opt -> test_fname = NULL;
    opt -> maxdeg = 30UL;
    opt -> maxiter = 20UL;
    opt -> norm = L_INF_NORM;
    opt -> print_weights = FALSE;
    opt -> lin_opt_step = (FLT_TYPE)0.01L;
    opt -> cache_data = TRUE;
    opt -> exact = FALSE;
    opt -> soft_auc = FALSE;
    opt -> beta = (FLT_TYPE)1;
    opt -> rumelhart = FALSE;
    opt -> laplace = FALSE;
    opt -> lambda = (FLT_TYPE)0;

    for(i = 1; i < argc; i++)
    {
	char * option = argv[i];

	if(option[0] == '-')
	{
	    if(!strcmp(option, "-d"))
	    {
		if(i == argc - 1) print_usage_and_exit(argv[0]);
		i++;
		opt -> maxdeg = atol(argv[i]);
	    }
	    else if(!strcmp(option, "-i"))
	    {
		if(i == argc - 1) print_usage_and_exit(argv[0]);
		i++;
		opt -> maxiter = atol(argv[i]);
	    }
	    else if(!strcmp(option, "-n"))
	    {
		if(i == argc - 1) print_usage_and_exit(argv[0]);
		i++;
		if(!strcmp(argv[i], "none")) opt -> norm = NO_NORM;
		else if(!strcmp(argv[i], "L1")) opt -> norm = L1_NORM;
		else if(!strcmp(argv[i], "L2")) opt -> norm = L2_NORM;
		else if(!strcmp(argv[i], "max")) opt -> norm = L_INF_NORM;
		else print_usage_and_exit(argv[0]);
	    }
	    else if(!strcmp(option, "-w"))
	    {
		opt -> print_weights = TRUE;
	    }
	    else if(!strcmp(option, "-c"))
	    {
		opt -> cache_data = FALSE;
	    }
	    else if(!strcmp(option, "-e"))
	    {
		opt -> exact = TRUE;
	    }
	    else if(!strcmp(option, "-f"))
	    {
		opt -> soft_auc = TRUE;
	    }
	    else if(!strcmp(option, "-b"))
	    {
		if(i == argc - 1) print_usage_and_exit(argv[0]);
		i++;
		sscanf(argv[i], FMT, &(opt -> beta));
	    }
	    else if(!strcmp(option, "-s"))
	    {
		if(i == argc - 1) print_usage_and_exit(argv[0]);
		i++;
		sscanf(argv[i], FMT, &(opt -> lin_opt_step));
	    }
	    else if(!strcmp(option, "-rr"))
	    {
		opt -> regul = TRUE;
		opt -> rumelhart = TRUE;
		if(i == argc - 1) print_usage_and_exit(argv[0]);
		i++;
		sscanf(argv[i], FMT, &(opt -> lambda));
	    }
	    else if(!strcmp(option, "-rl"))
	    {
		opt -> regul = TRUE;
		opt -> laplace = TRUE;
		if(i == argc - 1) print_usage_and_exit(argv[0]);
		i++;
		sscanf(argv[i], FMT, &(opt -> lambda));
	    }
	    else
	    {
		/* wrong option */
		print_usage_and_exit(argv[0]);
	    }
	}
	else
	{
	    /* assume file name */
	    if(opt -> train_fname == NULL)
	    {
		opt -> train_fname = option;
	    }
	    else if(opt -> test_fname == NULL)
	    {
		opt -> test_fname = option;
	    }
	    else
	    {
		/* too many file arguments */
		print_usage_and_exit(argv[0]);
	    }
	}
    }
    if(opt -> train_fname == NULL)
    {
	print_usage_and_exit(argv[0]);
    }
}
