#include "vect_utils.h"

void
vect_print(const FLT_TYPE * V, size_t n)
{
    size_t i;

    if(n > 0)
    {
	printf(PFMT, V[0]);
	for(i = 1; i < n; i++)
	{
	    printf(", "PFMT, V[i]);
	}
	printf("\n");
    }
}


FLT_TYPE
vect_check_orthogonality(const FLT_TYPE * weights, const FLT_TYPE * grad,
			 size_t nattrs)
{
    size_t i;
    FLT_TYPE scalar_prod = (FLT_TYPE)0;
    FLT_TYPE norm_grad = (FLT_TYPE)0;
    FLT_TYPE norm_weights = (FLT_TYPE)0;

    for(i = 0; i < nattrs; i++)
    {
	scalar_prod += weights[i] * grad[i];
	norm_grad += grad[i] * grad[i];
	norm_weights += weights[i] * weights[i];
    }
    return scalar_prod / (SQRT_F(norm_grad) * SQRT_F(norm_weights));
}

FLT_TYPE
vect_normalize(const FLT_TYPE * src, FLT_TYPE * dst, size_t size)
{
    size_t i;
    FLT_TYPE norm = (FLT_TYPE)0;

    for(i = 0; i < size; i++)
    {
	norm += src[i] * src[i];
    }
    norm = SQRT_F(norm);
    for(i = 0; i < size; i++)
    {
	dst[i] = src[i] / norm;
    }
    return norm;
}

void
vect_project_on_orthogonal_space(const FLT_TYPE * src,
				 const FLT_TYPE * norm_vect,
				 FLT_TYPE * projection,
				 size_t size)
{
    size_t i;
    FLT_TYPE scalar_prod = (FLT_TYPE)0;
    FLT_TYPE norm = (FLT_TYPE)0;

    for(i = 0; i < size; i++)
    {
	scalar_prod += src[i] * norm_vect[i];
	norm += norm_vect[i] * norm_vect[i];
    }
    scalar_prod /= norm;

    /* subtract the projection on norm_vect */
    for(i = 0; i < size; i++)
    {
	projection[i] = src[i] - scalar_prod * norm_vect[i];
    }
}




FLT_TYPE
compute_f(const FLT_TYPE * weights, const FLT_TYPE * X, size_t nattrs)
{
    FLT_TYPE f = (FLT_TYPE)0;
    size_t i;

    for(i = 0; i < nattrs; i++)
    {
	f += weights[i] * X[i];
    }
    return f;
}
